@@ -1,6 +1,6 @@
 package XML::MyXML;
 {
-  $XML::MyXML::VERSION = '0.0991';
+  $XML::MyXML::VERSION = '0.0992';
 }
 # ABSTRACT: A simple-to-use XML module, for parsing and creating XML documents
 
@@ -10,7 +10,7 @@ use utf8;
 use Carp;
 require Exporter;
 our @ISA = qw(Exporter);
-our @EXPORT_OK = qw(tidy_xml object_to_xml xml_to_object simple_to_xml xml_to_simple check_xml);
+our @EXPORT_OK = qw(tidy_xml object_to_xml xml_to_object simple_to_xml xml_to_simple check_xml ent_encode);
 our %EXPORT_TAGS = (all => [@EXPORT_OK]);
 use Encode;
 
@@ -31,6 +31,13 @@ sub _encode {
 	return $string;
 }
 
+
+sub ent_encode {
+	my ($string) = @_;
+
+	return _encode($string);
+}
+
 sub _decode {
 	my $string = shift;
 	my $entities = shift || {};
@@ -289,7 +296,11 @@ sub simple_to_xml {
 
 	if (! ref $value) {
 		Encode::_utf8_off($value);
-		$xml .= "<$key>"._encode($value)."</$tag>";
+		if (defined $value and length $value) {
+			$xml .= "<$key>"._encode($value)."</$tag>";
+		} else {
+			$xml .= "<$key/>";
+		}
 	} else {
 		$xml .= "<$key>"._arrayref_to_xml($value, $flags)."</$tag>";
 	}
@@ -333,7 +344,11 @@ sub _arrayref_to_xml {
 			$xml .= $value if check_xml($value);
 		} elsif (! ref $value) {
 			Encode::_utf8_off($value);
-			$xml .= "<$key>"._encode($value)."</$tag>";
+			if (defined $value and length $value) {
+				$xml .= "<$key>"._encode($value)."</$tag>";
+			} else {
+				$xml .= "<$key/>";
+			}
 		} else {
 			$xml .= "<$key>"._arrayref_to_xml($value, $flags)."</$tag>";
 		}
@@ -358,7 +373,11 @@ sub _hashref_to_xml {
 			$xml .= $value if check_xml($value);
 		} elsif (! ref $value) {
 			Encode::_utf8_off($value);
-			$xml .= "<$key>"._encode($value)."</$tag>";
+			if (defined $value and length $value) {
+				$xml .= "<$key>"._encode($value)."</$tag>";
+			} else {
+				$xml .= "<$key/>";
+			}
 		} else {
 			$xml .= "<$key>"._arrayref_to_xml($value, $flags)."</$tag>";
 		}
@@ -475,7 +494,7 @@ sub check_xml {
 
 package XML::MyXML::Object;
 {
-  $XML::MyXML::Object::VERSION = '0.0991';
+  $XML::MyXML::Object::VERSION = '0.0992';
 }
 
 use Carp;
@@ -490,17 +509,51 @@ sub new {
 	return $obj;
 }
 
+sub _parse_description {
+	my ($desc) = @_;
+
+	my ($tag, $attrs_str) = $desc =~ /^([^\[]*)(.*)$/g;
+	my %attrs = $attrs_str =~ /\[([^=\]]+)(?:=([^\]]*))?\]/g;
+
+	return ($tag, \%attrs);
+}
+
+sub cmp_element {
+	my ($self, $desc) = @_;
+
+	my ($tag, $attrs) = ref $desc
+			? @$desc{qw/ tag attrs /}
+			: _parse_description($desc);
+
+	! length $tag or $self->{'element'} =~ /(^|\:)\Q$tag\E$/	or return 0;
+	foreach my $attr (keys %$attrs) {
+		my $val = $self->attr($attr);
+		defined $val											or return 0;
+		! defined $attrs->{$attr} or $attrs->{$attr} eq $val	or return 0;
+	}
+
+	return 1;
+}
+
 sub children {
 	my $self = shift;
 	my $tag = shift;
 
 	$tag = '' if ! defined $tag;
 
-	if (length $tag) {
-		return grep {defined $_->{'element'} and $_->{'element'} =~ /(^|\:)\Q$tag\E$/} @{$self->{'content'}};
-	} else {
-		return grep { defined $_->{'element'} } @{$self->{'content'}};
+	my @all_children = grep { defined $_->{'element'} } @{$self->{'content'}};
+	length $tag		or return @all_children;
+
+	($tag, my $attrs) = _parse_description($tag);
+	my $desc = { tag => $tag, attrs => $attrs };
+
+	my @results;
+	CHILD: foreach my $child (@all_children) {
+		$child->cmp_element($desc)		or next;
+		push @results, $child;
 	}
+
+	return @results;
 }
 
 sub parent {
@@ -514,7 +567,19 @@ sub path {
 	my $self = shift;
 	my $path = shift;
 
-	my @path = split /\//, $path;
+	my @path;
+	my $orig_path = $path;
+	$path = "/" . $path;
+	while (length $path) {
+		my $success = $path =~ s!^/((?:[^/\[]*)?(?:\[[^\]]+\])*)!!;
+		my $seg = $1;
+		if ($success) {
+			push @path, $seg;
+		} else {
+			die "Invalid path: $orig_path";
+		}
+	}
+
 	my $el = $self;
 	for (my $i = 0; $i < $#path; $i++) {
 		my $pathstep = $path[$i];
@@ -543,12 +608,35 @@ sub value {
 sub attr {
 	my $self = shift;
 	my $attrname = shift;
-	my $flags = shift || {};
+	my ($set_to, $must_set, $flags);
+	if (@_) {
+		my $next = shift;
+		if (! ref $next) {
+			$set_to = $next;
+			Encode::_utf8_off($set_to);
+			$must_set = 1;
+			$flags = shift;
+		} else {
+			$flags = $next;
+		}
+	}
+	$flags ||= {};
 
 	if (defined $attrname) {
-		my $attrvalue = $self->{'attrs'}->{$attrname};
-		Encode::_utf8_on($attrvalue) if $flags->{'utf8'};
-		return $attrvalue;
+		if ($must_set) {
+			if (defined ($set_to)) {
+				$self->{'attrs'}{$attrname} = $set_to;
+				Encode::_utf8_on($set_to) if $flags->{'utf8'};
+				return $set_to;
+			} else {
+				delete $self->{'attrs'}{$attrname};
+				return;
+			}
+		} else {
+			my $attrvalue = $self->{'attrs'}->{$attrname};
+			Encode::_utf8_on($attrvalue) if $flags->{'utf8'};
+			return $attrvalue;
+		}
 	} else {
 		my %attr = %{$self->{'attrs'}};
 		if ($flags->{'utf8'}) {
@@ -567,7 +655,7 @@ sub tag {
 
 	my $tag = $self->{'element'};
 	if (defined $tag) {
-		$tag =~ s/^.*\://;
+		$tag =~ s/^.*\://	unless exists $flags->{'strip_ns'} and ! $flags->{'strip_ns'};
 		Encode::_utf8_on($tag) if $flags->{'utf8'};
 		return $tag;
 	} else {
@@ -665,7 +753,7 @@ XML::MyXML - A simple-to-use XML module, for parsing and creating XML documents
 
 =head1 VERSION
 
-version 0.0991
+version 0.0992
 
 =head1 SYNOPSIS
 
@@ -725,6 +813,10 @@ C<utf8> : the strings which will be returned will have their utf8 flag set (defa
 
 =head1 FUNCTIONS
 
+=head2 ent_encode($string)
+
+Returns the same string, but with the C<< < >>, C<< > >>, C<< & >>, C<< " >> and C<< ' >> characters replaced by their XML entities (e.g. C<< &amp; >>).
+
 =head2 tidy_xml($raw_xml)
 
 Returns the XML string in a tidy format (with tabs & newlines)
@@ -769,9 +861,9 @@ Optional flags: C<file>
 
 =head1 OBJECT METHODS
 
-=head2 $obj->path("subtag1/subsubtag2/.../subsubsubtagX")
+=head2 $obj->path("subtag1/subsubtag2[attr1=val1][attr2]/.../subsubsubtagX")
 
-Returns the element specified by the path as an XML::MyXML::Object object. When there are more than one tags with the specified name in the last step of the path, it will return all of them as an array. In scalar context will only return the first one.
+Returns the element specified by the path as an XML::MyXML::Object object. When there are more than one tags with the specified name in the last step of the path, it will return all of them as an array. In scalar context will only return the first one. CSS3-style attribute selectors are allowed in the path next to the tagnames, for example: C<< p[class=big] >> will only return C<< <p> >> elements that contain an attribute called "class" with a value of "big". p[class] on the other hand will return p elements having a "class" attribute, but that attribute can have any value.
 
 =head2 $obj->value
 
@@ -779,18 +871,18 @@ When the element represented by the $obj object has only text contents, returns
 
 Optional flags: C<strip>, C<utf8>
 
-=head2 $obj->attr('attrname')
+=head2 $obj->attr('attrname' [, 'attrvalue'])
 
-Returns the value of the 'attrname' attribute of the top element. Returns undef if attribute does not exist. If called without the 'attrname' paramter, returns a hash with all attribute => value pairs.
+Gets/Sets the value of the 'attrname' attribute of the top element. Returns undef if attribute does not exist. If called without the 'attrname' paramter, returns a hash with all attribute => value pairs. If setting with an attrvalue of C<undef>, then removes that attribute entirely.
 
 Optional flags: C<utf8>
 
 =head2 $obj->tag
 
-Returns the tag of the $obj element (after stripping it from namespaces). E.g. if $obj represents an <rss:item> element, C<< $obj->tag >> will just return the name 'item'.
+Returns the tag of the $obj element (after stripping it from namespaces, unless the C<strip_ns> option is passed as false). E.g. if $obj represents an <rss:item> element, C<< $obj->tag >> will just return the name 'item'.
 Returns undef if $obj doesn't represent a tag.
 
-Optional flags: C<utf8>
+Optional flags: C<utf8>, C<strip_ns>
 
 =head2 $obj->simplify
 
